---
permalink: /404.html
layout: page
---

<style type="text/css" media="screen">
  .container {
    margin: 10px auto;
    max-width: 600px;
    text-align: center;
  }
  h1 {
    margin: 30px 0;
    font-size: 4em;
    line-height: 1;
    letter-spacing: -1px;
  }
</style>

<div class="container">
  <h1>404</h1>

  <p align="center">
    <img src="{{site.baseurl}}/images/treadi_logo.svg" alt="logo life ring resembling github issue icon" width="256"/>
  </p>
  <p><strong>Page not found :(</strong></p>
</div>
